// Copyright 2000-2005 the Contributors, as shown in the revision logs.
// Licensed under the Apache Public Source License 2.0 ("the License").
// You may not use this file except in compliance with the License.

package org.ibex.js; 

/** The JavaScript Math object */
class JSMath extends JS.Immutable {
    private static final JS.Method METHOD = new JS.Method();

    private static final JS E       = Script.N(java.lang.Math.E);
    private static final JS PI      = Script.N(java.lang.Math.PI);
    private static final JS LN10    = Script.N(java.lang.Math.log(10));
    private static final JS LN2     = Script.N(java.lang.Math.log(2));
    private static final JS LOG10E  = Script.N(1/java.lang.Math.log(10));
    private static final JS LOG2E   = Script.N(1/java.lang.Math.log(2));
    private static final JS SQRT1_2 = Script.N(1/java.lang.Math.sqrt(2));
    private static final JS SQRT2   = Script.N(java.lang.Math.sqrt(2));

    public JS call(JS method, JS[] args) throws JSExn {
        switch(args.length) {
            case 0: {
                //#switch(Script.toString(method))
                case "random": return Script.N(java.lang.Math.random());
                //#end
                break;
            }
            case 1: {
                //#switch(Script.toString(method))
                case "ceil": return Script.N((long)java.lang.Math.ceil(Script.toDouble(args[0])));
                case "floor": return Script.N((long)java.lang.Math.floor(Script.toDouble(args[0])));
                case "round": return Script.N((long)java.lang.Math.round(Script.toDouble(args[0])));
                case "abs": return Script.N(java.lang.Math.abs(Script.toDouble(args[0])));
                case "sin": return Script.N(java.lang.Math.sin(Script.toDouble(args[0])));
                case "cos": return Script.N(java.lang.Math.cos(Script.toDouble(args[0])));
                case "tan": return Script.N(java.lang.Math.tan(Script.toDouble(args[0])));
                case "asin": return Script.N(java.lang.Math.asin(Script.toDouble(args[0])));
                case "acos": return Script.N(java.lang.Math.acos(Script.toDouble(args[0])));
                case "atan": return Script.N(java.lang.Math.atan(Script.toDouble(args[0])));
                case "sqrt": return Script.N(java.lang.Math.sqrt(Script.toDouble(args[0])));
                case "exp": return Script.N(java.lang.Math.exp(Script.toDouble(args[0])));
                case "log": return Script.N(java.lang.Math.log(Script.toDouble(args[0])));
                //#end
                break;
            }
            case 2: {
                //#switch(Script.toString(method))
                case "min": return Script.N(java.lang.Math.min(Script.toDouble(args[0]), Script.toDouble(args[1])));
                case "max": return Script.N(java.lang.Math.max(Script.toDouble(args[0]), Script.toDouble(args[1])));
                case "pow": return Script.N(java.lang.Math.pow(Script.toDouble(args[0]), Script.toDouble(args[1])));
                case "atan2": return Script.N(java.lang.Math.atan2(Script.toDouble(args[0]), Script.toDouble(args[1])));
                //#end
                break;
            }
        }
        return super.call(method, args);
    }

    public JS get(JS key) throws JSExn {
        //#switch(Script.toString(key))
        case "E": return E;
        case "LN10": return LN10;
        case "LN2": return LN2;
        case "LOG10E": return LOG10E;
        case "LOG2E": return LOG2E;
        case "PI": return PI;
        case "SQRT1_2": return SQRT1_2;
        case "SQRT2": return SQRT2;
        case "ceil": return METHOD;
        case "floor": return METHOD;
        case "round": return METHOD;
        case "min": return METHOD;
        case "max": return METHOD;
        case "pow": return METHOD;
        case "atan2": return METHOD;
        case "abs": return METHOD;
        case "sin": return METHOD;
        case "cos": return METHOD;
        case "tan": return METHOD;
        case "asin": return METHOD;
        case "acos": return METHOD;
        case "atan": return METHOD;
        case "sqrt": return METHOD;
        case "exp": return METHOD;
        case "log": return METHOD;
        case "random": return METHOD;
        //#end
        return super.get(key);
    }
}
